from dymos.models.atmosphere.atmos_1976 import USatm1976Comp
from aviary.utils.aviary_values import AviaryValues
from aviary.utils.functions import add_opts2vals, create_opts2vals

from aviary.mission.gasp_based.ode.base_ode import BaseODE
from aviary.mission.gasp_based.ode.params import ParamPort
from aviary.mission.gasp_based.phases.taxi_component import TaxiFuelComponent
from aviary.subsystems.propulsion.propulsion_mission import \
    PropulsionMission
from aviary.variable_info.variables import Dynamic, Mission


class TaxiSegment(BaseODE):
    def setup(self):
        options: AviaryValues = self.options['aviary_options']
        self.add_subsystem("params", ParamPort(), promotes=["*"])
        self.add_subsystem(
            "USatm",
            USatm1976Comp(num_nodes=1),
            promotes_inputs=[("h", Mission.Takeoff.AIRPORT_ALTITUDE)],
            promotes_outputs=[("temp", Dynamic.Mission.TEMPERATURE),
                              ("pres", Dynamic.Mission.STATIC_PRESSURE)],
        )

        add_opts2vals(self, create_opts2vals(
            [Mission.Taxi.MACH]), options)

        self.add_subsystem(
            name='propulsion',
            subsys=PropulsionMission(
                num_nodes=1,
                aviary_options=options,
            ),
            promotes_inputs=['*', (Dynamic.Mission.ALTITUDE, Mission.Takeoff.AIRPORT_ALTITUDE),
                             (Dynamic.Mission.MACH, Mission.Taxi.MACH)],
            promotes_outputs=['*'])

        self.add_subsystem("taxifuel", TaxiFuelComponent(
            aviary_options=options), promotes=["*"])

        ParamPort.set_default_vals(self)
        self.set_input_defaults(Mission.Taxi.MACH, 0)

        # Throttle Idle
        self.set_input_defaults('throttle', 0.0)



import unittest

from copy import deepcopy

import openmdao.api as om

import aviary.api as av
from aviary.subsystems.subsystem_builder_base import SubsystemBuilderBase
from aviary.subsystems.test.subsystem_tester import TestSubsystemBuilderBase
from aviary.variable_info.variables import Aircraft as av_Aircraft
from aviary.variable_info.variables import Mission as av_Mission


class Aircraft(av_Aircraft):
    class Dummy:
        VARIABLE = "aircraft:dummy:VARIABLE"
        VARIABLE_OUT = "aircraft:dummy:VARIABLE_OUT"
        PARAMETER = "aircraft:dummy:PARAMETER"


class Mission(av_Mission):
    class Dummy:
        VARIABLE = "mission:dummy:VARIABLE"
        VARIABLE_RATE = "mission:dummy:VARIABLE_RATE"


class MoreAircraft(av_Aircraft):
    class Dummy:
        DUMMY_WINGSPAN = "aircraft:dummy:DUMMY_WINGSPAN"
        DUMMY_AIRSPEED = "aircraft:dummy:DUMMY_AIRSPEED"
        DUMMY_FUEL_CAPACITY = "aircraft:dummy:DUMMY_FUEL_CAPACITY"


class MoreMission(av_Mission):
    class Dummy:
        DUMMY_FLIGHT_DURATION = "mission:dummy:DUMMY_FLIGHT_DURATION"
        DUMMY_TAKEOFF_WEIGHT = "mission:dummy:DUMMY_TAKEOFF_WEIGHT"
        DUMMY_CONTROL = "mission:dummy:DUMMY_CONTROL"
        TIMESERIES_VAR = "mission:dummy:TIMESERIES_VAR"


ExtendedMetaData = deepcopy(av.CoreMetaData)
AdditionalMetaData = deepcopy(av.CoreMetaData)

# Variables for ExtendedMetaData
av.add_meta_data(
    Aircraft.Dummy.VARIABLE,
    desc="Dummy aircraft variable",
    default_value=0.5,
    option=False,
    units='kn',
    meta_data=ExtendedMetaData
)

av.add_meta_data(
    Aircraft.Dummy.VARIABLE_OUT,
    desc="Dummy aircraft variable out",
    default_value=0.5,
    option=False,
    units='kg',
    meta_data=ExtendedMetaData
)

av.add_meta_data(
    Aircraft.Dummy.PARAMETER,
    desc="Dummy mission parameter",
    default_value=0.5,
    option=False,
    units='m',
    meta_data=ExtendedMetaData
)

av.add_meta_data(
    Mission.Dummy.VARIABLE,
    desc="Dummy mission variable",
    default_value=0.5,
    option=False,
    units='m',
    meta_data=ExtendedMetaData
)

av.add_meta_data(
    Mission.Dummy.VARIABLE_RATE,
    desc="Dummy mission variable rate",
    default_value=0.5,
    option=False,
    units='m/s',
    meta_data=ExtendedMetaData
)

# Variables for AdditionalMetaData
av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_WINGSPAN,
    desc="Dummy wingspan variable",
    default_value=35.0,
    option=False,
    units='m',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_WINGSPAN+'_out',
    desc="Output of dummy wingspan variable",
    default_value=70.0,
    option=False,
    units='m',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_AIRSPEED,
    desc="Dummy airspeed variable",
    default_value=500.0,
    option=False,
    units='kn',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_AIRSPEED+'_out',
    desc="Output of dummy airspeed variable",
    default_value=1000.0,
    option=False,
    units='kn',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_FUEL_CAPACITY,
    desc="Dummy fuel capacity variable",
    default_value=20000.0,
    option=False,
    units='kg',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreAircraft.Dummy.DUMMY_FUEL_CAPACITY+'_out',
    desc="Output of dummy fuel capacity variable",
    default_value=40000.0,
    option=False,
    units='kg',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.DUMMY_FLIGHT_DURATION,
    desc="Dummy flight duration variable",
    default_value=6.0,
    option=False,
    units='hr',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.DUMMY_FLIGHT_DURATION+'_rate',
    desc="Rate of dummy flight duration variable",
    default_value=12.0,
    option=False,
    units='hr/s',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.DUMMY_TAKEOFF_WEIGHT,
    desc="Dummy takeoff weight variable",
    default_value=80000.0,
    option=False,
    units='kg',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.DUMMY_TAKEOFF_WEIGHT+'_rate',
    desc="Rate of dummy takeoff weight variable",
    default_value=160000.0,
    option=False,
    units='kg/s',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.DUMMY_CONTROL,
    desc="Dummy control variable",
    default_value=0.5,
    option=False,
    units='unitless',
    meta_data=AdditionalMetaData
)

av.add_meta_data(
    MoreMission.Dummy.TIMESERIES_VAR,
    desc="Dummy timeseries variable",
    default_value=0.5,
    option=False,
    units='unitless',
    meta_data=AdditionalMetaData
)


class DummyComp(om.ExplicitComponent):
    def setup(self):
        self.add_input(Aircraft.Dummy.VARIABLE, units='kn')
        self.add_output(Aircraft.Dummy.VARIABLE_OUT, units='kg')

        self.declare_partials('*', '*', method='fd')

    def compute(self, inputs, outputs):
        outputs[Aircraft.Dummy.VARIABLE_OUT] = 2 * inputs[Aircraft.Dummy.VARIABLE]


class DummyMissionComp(om.ExplicitComponent):
    def initialize(self):
        self.options.declare('num_nodes', types=int)

    def setup(self):
        nn = self.options['num_nodes']
        self.add_input(Mission.Dummy.VARIABLE, units='m', shape=nn)
        self.add_input(Aircraft.Dummy.PARAMETER, units='m')
        self.add_output(Mission.Dummy.VARIABLE_RATE, units='m/s', shape=nn)

        self.declare_partials('*', '*', method='fd')

    def compute(self, inputs, outputs):
        outputs[Mission.Dummy.VARIABLE_RATE] = 2 * \
            inputs[Mission.Dummy.VARIABLE] + inputs[Aircraft.Dummy.PARAMETER] * 0.5


class PreOnlyBuilder(SubsystemBuilderBase):
    def build_pre_mission(self, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', DummyComp(), promotes=['*'])
        return group

    def get_mass_names(self):
        return [Aircraft.Dummy.VARIABLE_OUT]


class PostOnlyBuilder(SubsystemBuilderBase):
    def build_post_mission(self, aviary_values):
        group = om.Group()
        group.add_subsystem('comp', om.ExecComp('y = x**2'), promotes=['*'])
        return group


class FailingSubsystemBuilder(SubsystemBuilderBase):
    def get_states(self):
        return {
            'State1': {
                'rate_source': 'NonExistentRateSource',
            }
        }

    def build_mission(self, num_nodes, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', om.ExecComp('y = x**2'))
        return group


class ArrayGuessSubsystemBuilder(SubsystemBuilderBase):
    def __init__(self, name='array_guess'):
        super().__init__(name, meta_data=ExtendedMetaData)

    def build_pre_mission(self, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', DummyComp(), promotes=['*'])
        return group

    def build_mission(self, num_nodes, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', DummyMissionComp(
            num_nodes=num_nodes), promotes=['*'])
        return group

    def get_initial_guesses(self):
        return {
            Mission.Dummy.VARIABLE: {
                'val': [1.0, 2.0, 3.0],
                'type': 'state',
                'units': 'm',
            }
        }

    def get_states(self):
        return {
            Mission.Dummy.VARIABLE: {
                'rate_source': Mission.Dummy.VARIABLE_RATE,
                'units': 'm',
            }
        }

    def get_controls(self):
        return {}

    def get_parameters(self):
        return {Aircraft.Dummy.PARAMETER: {'val': 2., 'units': 'm'}}


class DummyWingspanComp(om.ExplicitComponent):
    def setup(self):
        self.add_input(MoreAircraft.Dummy.DUMMY_WINGSPAN, units='m')
        self.add_output(MoreAircraft.Dummy.DUMMY_WINGSPAN+'_out', units='m')

        self.declare_partials('*', '*', method='fd')

    def compute(self, inputs, outputs):
        outputs[MoreAircraft.Dummy.DUMMY_WINGSPAN+'_out'] = 2 * \
            inputs[MoreAircraft.Dummy.DUMMY_WINGSPAN]


class DummyFlightDurationComp(om.ExplicitComponent):
    def initialize(self):
        self.options.declare('num_nodes', types=int)

    def setup(self):
        nn = self.options['num_nodes']
        self.add_input(MoreMission.Dummy.DUMMY_FLIGHT_DURATION, units='h', shape=nn)
        self.add_input(MoreMission.Dummy.DUMMY_CONTROL, units='unitless', shape=nn)
        self.add_output(MoreMission.Dummy.DUMMY_FLIGHT_DURATION +
                        '_rate', units='h/s', shape=nn)
        self.add_output(MoreMission.Dummy.TIMESERIES_VAR, units='unitless', shape=nn)

        self.declare_partials('*', '*', method='fd')

    def compute(self, inputs, outputs):
        outputs[MoreMission.Dummy.DUMMY_FLIGHT_DURATION+'_rate'] = 2 * \
            inputs[MoreMission.Dummy.DUMMY_FLIGHT_DURATION] * \
            inputs[MoreMission.Dummy.DUMMY_CONTROL]
        outputs[MoreMission.Dummy.TIMESERIES_VAR] = inputs[MoreMission.Dummy.DUMMY_CONTROL] ** 2 + 0.5


class AdditionalArrayGuessSubsystemBuilder(SubsystemBuilderBase):
    def __init__(self, name='additional_array_guess'):
        super().__init__(name, meta_data=AdditionalMetaData)

    def build_pre_mission(self, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', DummyWingspanComp(), promotes=['*'])
        return group

    def build_mission(self, num_nodes, aviary_inputs):
        group = om.Group()
        group.add_subsystem('comp', DummyFlightDurationComp(
            num_nodes=num_nodes), promotes=['*'])
        return group

    def get_initial_guesses(self):
        return {
            MoreMission.Dummy.DUMMY_FLIGHT_DURATION: {
                'val': [1.0, 2.0, 3.0],
                'type': 'state',
                'units': 'h',
            }
        }

    def get_states(self):
        return {
            MoreMission.Dummy.DUMMY_FLIGHT_DURATION: {
                'rate_source': MoreMission.Dummy.DUMMY_FLIGHT_DURATION+'_rate',
                'units': 'h',
            }
        }

    def get_controls(self, phase_name=None):
        if phase_name == "cruise":
            controls_dict = {
                MoreMission.Dummy.DUMMY_CONTROL: {
                    'units': 'unitless',
                    'opt': True,
                    'lower': 0,
                    'upper': 1,
                }
            }
        else:
            controls_dict = {}
        return controls_dict

    def get_outputs(self):
        return [MoreMission.Dummy.DUMMY_CONTROL, MoreMission.Dummy.TIMESERIES_VAR]


class TestPreOnly(TestSubsystemBuilderBase):
    def setUp(self):
        self.subsystem_builder = PreOnlyBuilder()


class TestPostOnly(TestSubsystemBuilderBase):
    def setUp(self):
        self.subsystem_builder = PostOnlyBuilder()


class TestFailingBuilder(TestSubsystemBuilderBase):
    def setUp(self):
        self.subsystem_builder = FailingSubsystemBuilder()

    def test_check_state_variables(self):
        with self.assertRaises(AssertionError):
            super().test_check_state_variables()


class TestArrayGuessBuilder(TestSubsystemBuilderBase):
    def setUp(self):
        self.subsystem_builder = ArrayGuessSubsystemBuilder()


class TestAdditionalArrayGuessBuilder(TestSubsystemBuilderBase):
    def setUp(self):
        self.subsystem_builder = AdditionalArrayGuessSubsystemBuilder()


if __name__ == '__main__':
    unittest.main()



import unittest

import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials, assert_near_equal

from aviary.subsystems.geometry.gasp_based.fuselage import (FuselageGroup,
                                                            FuselageParameters,
                                                            FuselageSize)
from aviary.variable_info.options import get_option_defaults
from aviary.variable_info.variables import Aircraft


class FuselageParametersTestCase1(
    unittest.TestCase
):  # this is the GASP test case, input and output values based on large single aisle 1 v3 without bug fix
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=180, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24, units="inch")
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 6)
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29, units="inch")
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2, units="inch")

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "parameters",
            FuselageParameters(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob[Aircraft.Fuselage.AVG_DIAMETER], 157.2, tol)
        assert_near_equal(self.prob["cabin_height"], 13.1, tol)
        assert_near_equal(self.prob["cabin_len"], 72.1, tol)
        assert_near_equal(self.prob["nose_height"], 8.6, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


class FuselageParametersTestCase2(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=30, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24, units="inch")
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 1)
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29, units="inch")
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2, units="inch")

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "parameters",
            FuselageParameters(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case2(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob[Aircraft.Fuselage.AVG_DIAMETER], 56.2, tol)
        assert_near_equal(self.prob["cabin_height"], 9.183, tol)  # not actual GASP value
        assert_near_equal(self.prob["cabin_len"], 72.5, tol)  # not actual GASP value
        assert_near_equal(self.prob["nose_height"], 4.683, tol)  # not actual GASP value

        partial_data2 = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data2, atol=1e-8, rtol=1e-8)


class FuselageSizeTestCase1(
    unittest.TestCase
):  # this is the GASP test case, input and output values based on large single aisle 1 v3 without bug fix
    def setUp(self):

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "size", FuselageSize(aviary_options=get_option_defaults()), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless")
        self.prob.model.set_input_defaults("nose_height", 8.6, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft")
        self.prob.model.set_input_defaults("cabin_len", 72.1, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless")
        self.prob.model.set_input_defaults("cabin_height", 13.1, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 4000, units="unitless")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 129.5, tol
        )  # note: this is the actual GASP value, but for version 3.5. Version 3 has 129.4
        assert_near_equal(self.prob[Aircraft.Fuselage.WETTED_AREA], 4000, tol)
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 129.5, tol
        )  # note: this is the actual GASP value, but for version 3.5. Version 3 has 129.4

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


class FuselageSizeTestCase2(
    unittest.TestCase
):  # this is the GASP test case for V3.6 advanced tube and wing
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Fuselage.PROVIDE_SURFACE_AREA,
                        val=False, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "parameters", FuselageSize(aviary_options=options), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless")
        self.prob.model.set_input_defaults("nose_height", 8.6, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft")
        self.prob.model.set_input_defaults("cabin_len", 61.6, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless")
        self.prob.model.set_input_defaults("cabin_height", 13.1, units="ft")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 1, units="unitless")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case2(self):

        self.prob.run_model()

        tol = 3e-4
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 119.03, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.WETTED_AREA], 4209, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 119.03, tol
        )  # not actual GASP value

        partial_data2 = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data2, atol=1e-8, rtol=1e-8)


class FuselageGroupTestCase1(
    unittest.TestCase
):  # this is the GASP test case, input and output values based on large single aisle 1 v3 without bug fix
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=180, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24, units="inch")
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 6)
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29, units="inch")
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2, units="inch")

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "group",
            FuselageGroup(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft")

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless")
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 4000, units="unitless")

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob[Aircraft.Fuselage.AVG_DIAMETER], 157.2, tol)
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 129.5, tol
        )  # note: this is the actual GASP value, but for version 3.5. Version 3 has 129.4
        assert_near_equal(self.prob[Aircraft.Fuselage.WETTED_AREA], 4000, tol)
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 129.5, tol
        )  # note: this is the actual GASP value, but for version 3.5. Version 3 has 129.4

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


class FuselageGroupTestCase2(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Fuselage.PROVIDE_SURFACE_AREA,
                        val=False, units='unitless')
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=180, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 6)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2,
                        units="inch")  # not actual GASP value

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "group",
            FuselageGroup(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 4000, units="unitless"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft"
        )  # not actual GASP value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob[Aircraft.Fuselage.AVG_DIAMETER], 157.2, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 129.5, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.WETTED_AREA], 18558260.55555555, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 129.5, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


class FuselageGroupTestCase3(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Fuselage.PROVIDE_SURFACE_AREA,
                        val=False, units='unitless')
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=30, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 1)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2,
                        units="inch")  # not actual GASP value

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "group",
            FuselageGroup(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 4000, units="unitless"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft"
        )  # not actual GASP value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob[Aircraft.Fuselage.AVG_DIAMETER], 56.2, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 114.23, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.WETTED_AREA], 1.1790053e7, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 114.23, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


class FuselageGroupTestCase4(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, val=30, units='unitless')
        options.set_val(Aircraft.Fuselage.AISLE_WIDTH, 24,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_AISLES, 1)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.NUM_SEATS_ABREAST, 1)  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_PITCH, 29,
                        units="inch")  # not actual GASP value
        options.set_val(Aircraft.Fuselage.SEAT_WIDTH, 20.2,
                        units="inch")  # not actual GASP value

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "group",
            FuselageGroup(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.DELTA_DIAMETER, 4.5, units="ft"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.NOSE_FINENESS, 1, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.TAIL_FINENESS, 3, units="unitless"
        )  # not actual GASP value
        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.WETTED_AREA_FACTOR, 4000, units="unitless"
        )  # not actual GASP value

        self.prob.model.set_input_defaults(
            Aircraft.Fuselage.PILOT_COMPARTMENT_LENGTH, 9.5, units="ft"
        )  # not actual GASP value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob[Aircraft.Fuselage.AVG_DIAMETER], 56.2, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.LENGTH], 114.23, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.Fuselage.WETTED_AREA], 4000, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob[Aircraft.TailBoom.LENGTH], 114.23, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


if __name__ == "__main__":
    unittest.main()



import numpy as np
import openmdao.api as om

from aviary.constants import GRAV_ENGLISH_LBM
from aviary.subsystems.mass.flops_based.distributed_prop import (
    distributed_engine_count_factor, distributed_nacelle_diam_factor)
from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.functions import add_aviary_input, add_aviary_output
from aviary.variable_info.variables import Aircraft


class AntiIcingMass(om.ExplicitComponent):
    '''
    Calculates the mass of the anti-icing system. The methodology is based
    on the FLOPS weight equations, modified to output mass instead of weight.
    '''

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.AntiIcing.MASS_SCALER, val=1.0)

        add_aviary_input(self, Aircraft.Fuselage.MAX_WIDTH, val=0.0)

        add_aviary_input(self, Aircraft.Nacelle.AVG_DIAMETER, val=0.0)

        add_aviary_input(self, Aircraft.Wing.SPAN, val=0.0)

        add_aviary_input(self, Aircraft.Wing.SWEEP, val=0.0)

        add_aviary_output(self, Aircraft.AntiIcing.MASS, val=0.0)

    def setup_partials(self):
        self.declare_partials("*", "*")

    def compute(self, inputs, outputs):
        aviary_options: AviaryValues = self.options['aviary_options']
        total_engines = aviary_options.get_val(Aircraft.Propulsion.TOTAL_NUM_ENGINES)
        num_engines = aviary_options.get_val(Aircraft.Engine.NUM_ENGINES)

        scaler = inputs[Aircraft.AntiIcing.MASS_SCALER]
        max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
        avg_diam = inputs[Aircraft.Nacelle.AVG_DIAMETER]
        span = inputs[Aircraft.Wing.SPAN]
        sweep = inputs[Aircraft.Wing.SWEEP]

        count_factor = distributed_engine_count_factor(total_engines)
        f_nacelle = distributed_nacelle_diam_factor(avg_diam, num_engines)

        outputs[Aircraft.AntiIcing.MASS] = (
            (span / np.cos(sweep * np.pi / 180))
            + 3.8 * f_nacelle * count_factor + 1.5 * max_width) * scaler / GRAV_ENGLISH_LBM

    def compute_partials(self, inputs, J):
        aviary_options: AviaryValues = self.options['aviary_options']
        total_engines = aviary_options.get_val(Aircraft.Propulsion.TOTAL_NUM_ENGINES)
        num_engines = aviary_options.get_val(Aircraft.Engine.NUM_ENGINES)

        scaler = inputs[Aircraft.AntiIcing.MASS_SCALER]
        max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
        avg_diam = inputs[Aircraft.Nacelle.AVG_DIAMETER]
        count_factor = distributed_engine_count_factor(total_engines)
        span = inputs[Aircraft.Wing.SPAN]
        sweep = inputs[Aircraft.Wing.SWEEP]

        weighted_avg_diam = sum(avg_diam * num_engines) / total_engines

        diam_deriv_fact = 1.0
        if total_engines > 4:
            diam_deriv_fact = 0.5 * total_engines ** 0.5

        f_nacelle = weighted_avg_diam * diam_deriv_fact

        cos_sweep = np.cos(sweep * np.pi / 180)
        sin_sweep = np.sin(sweep * np.pi / 180)

        J[Aircraft.AntiIcing.MASS, Aircraft.AntiIcing.MASS_SCALER] = (
            span / cos_sweep + 3.8 * f_nacelle * count_factor +
            1.5 * max_width) / GRAV_ENGLISH_LBM

        J[Aircraft.AntiIcing.MASS, Aircraft.Fuselage.MAX_WIDTH] = \
            1.5 * scaler / GRAV_ENGLISH_LBM

        J[Aircraft.AntiIcing.MASS, Aircraft.Nacelle.AVG_DIAMETER] = \
            3.8 * count_factor * diam_deriv_fact * scaler / GRAV_ENGLISH_LBM

        J[Aircraft.AntiIcing.MASS, Aircraft.Wing.SPAN] = \
            1 / cos_sweep * scaler / GRAV_ENGLISH_LBM

        J[Aircraft.AntiIcing.MASS, Aircraft.Wing.SWEEP] = \
            span * (np.pi / 180) * sin_sweep / (cos_sweep) ** 2 * \
            scaler / GRAV_ENGLISH_LBM



"""
Contains any preliminary calculations on the wing.
"""

import openmdao.api as om

from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.functions import add_aviary_input, add_aviary_output
from aviary.variable_info.variables import Aircraft


class WingPrelim(om.ExplicitComponent):

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Wing.AREA, val=0.0)
        add_aviary_input(self, Aircraft.Wing.GLOVE_AND_BAT, val=0.0)
        add_aviary_input(self, Aircraft.Wing.SPAN, val=0.0)

        add_aviary_output(self, Aircraft.Wing.ASPECT_RATIO, val=0.0)

    def setup_partials(self):
        self.declare_partials('*', '*')

    def compute(self, inputs, outputs):
        area = inputs[Aircraft.Wing.AREA]
        glove_and_bat = inputs[Aircraft.Wing.GLOVE_AND_BAT]
        span = inputs[Aircraft.Wing.SPAN]

        AR = span ** 2 / (area - glove_and_bat)
        outputs[Aircraft.Wing.ASPECT_RATIO] = AR

    def compute_partials(self, inputs, partials, discrete_inputs=None):
        area = inputs[Aircraft.Wing.AREA]
        glove_and_bat = inputs[Aircraft.Wing.GLOVE_AND_BAT]
        span = inputs[Aircraft.Wing.SPAN]

        denom = 1.0 / (area - glove_and_bat)

        partials[Aircraft.Wing.ASPECT_RATIO,
                 Aircraft.Wing.AREA] = -(span * denom) ** 2

        partials[Aircraft.Wing.ASPECT_RATIO,
                 Aircraft.Wing.GLOVE_AND_BAT] = (span * denom) ** 2

        partials[Aircraft.Wing.ASPECT_RATIO,
                 Aircraft.Wing.SPAN] = 2.0 * span * denom



import unittest
import os

import numpy as np
import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials

from aviary.mission.gasp_based.ode.climb_ode import ClimbODE
from aviary.utils.test_utils.IO_test_util import check_prob_outputs
from aviary.variable_info.options import get_option_defaults
from aviary.variable_info.variables import Aircraft, Dynamic
from aviary.interface.default_phase_info.two_dof import default_mission_subsystems


class ClimbODETestCase(unittest.TestCase):
    def setUp(self):
        self.prob = om.Problem()
        self.sys = self.prob.model = ClimbODE(
            num_nodes=1,
            EAS_target=250,
            mach_cruise=0.8,
            aviary_options=get_option_defaults(),
            core_subsystems=default_mission_subsystems
        )

    def test_start_of_climb(self):
        """Test against GASP start of climb at 250 kts EAS, check partials"""
        self.sys.options["EAS_target"] = 250

        self.prob.setup(check=False, force_alloc_complex=True)

        throttle_climb = 0.956
        self.prob.set_val(
            Dynamic.Mission.THROTTLE, throttle_climb, units='unitless')
        self.prob.set_val(Dynamic.Mission.ALTITUDE, 1000, units="ft")
        self.prob.set_val(Dynamic.Mission.MASS, 174845, units="lbm")
        self.prob.set_val("EAS", 250, units="kn")
        # slightly greater than zero to help check partials
        self.prob.set_val(Aircraft.Wing.INCIDENCE, 0.0000001, units="deg")

        self.prob.run_model()

        testvals = {
            "alpha": 5.19,
            "CL": 0.5975,
            "CD": 0.0307,
            Dynamic.Mission.ALTITUDE_RATE: 3186 / 60,
            # TAS (kts -> ft/s) * cos(gamma)
            Dynamic.Mission.DISTANCE_RATE: (254 * 1.68781) * np.cos(np.deg2rad(7.12)),
            Dynamic.Mission.FUEL_FLOW_RATE_NEGATIVE_TOTAL: -13505,  # lbm/h
            "theta": np.deg2rad(12.31),
            Dynamic.Mission.FLIGHT_PATH_ANGLE: np.deg2rad(7.12),
        }
        check_prob_outputs(self.prob, testvals, rtol=1e-1)  # TODO tighten

        self.prob.setup(check=False, force_alloc_complex=True)
        partial_data = self.prob.check_partials(
            out_stream=None, method="cs", excludes=["*USatm*", "*params*", "*aero*"]
        )
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)

    def test_end_of_climb(self):
        """Test against GASP at 270 kts EAS and at cruise Mach."""
        self.sys.options["num_nodes"] = 2
        self.sys.options["EAS_target"] = 270

        self.prob.setup(check=False, force_alloc_complex=True)

        throttle_climb = 0.956
        self.prob.set_val(
            Dynamic.Mission.THROTTLE, np.array([
                throttle_climb, throttle_climb]), units='unitless')
        self.prob.set_val(Dynamic.Mission.ALTITUDE, np.array([11000, 37000]), units="ft")
        self.prob.set_val(Dynamic.Mission.MASS, np.array([174149, 171592]), units="lbm")
        self.prob.set_val("EAS", np.array([270, 270]), units="kn")

        self.prob.run_model()

        testvals = {
            "alpha": [4.08, 4.05],
            "CL": [0.5119, 0.6113],
            "CD": [0.0270, 0.0326],
            Dynamic.Mission.ALTITUDE_RATE: [3054 / 60, 453 / 60],
            # TAS (kts -> ft/s) * cos(gamma)
            Dynamic.Mission.DISTANCE_RATE: [
                (319 * 1.68781) * np.cos(np.deg2rad(5.42)),
                (459 * 1.68781) * np.cos(np.deg2rad(0.56)),
            ],
            Dynamic.Mission.FUEL_FLOW_RATE_NEGATIVE_TOTAL: [-11422, -6039],
            "theta": np.deg2rad([9.5, 4.61]),
            Dynamic.Mission.FLIGHT_PATH_ANGLE: np.deg2rad([5.42, 0.56]),
            Dynamic.Mission.THRUST_TOTAL: [25610, 10790],
        }
        check_prob_outputs(self.prob, testvals, 1e-1)  # TODO tighten


if __name__ == "__main__":
    unittest.main()



import unittest
import os

import numpy as np
import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials

from aviary.mission.gasp_based.ode.rotation_eom import RotationEOM
from aviary.variable_info.variables import Aircraft, Dynamic


class RotationEOMTestCase(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()
        self.prob.model.add_subsystem("group", RotationEOM(num_nodes=2), promotes=["*"])
        self.prob.model.set_input_defaults(
            Dynamic.Mission.MASS, val=175400 * np.ones(2), units="lbm"
        )
        self.prob.model.set_input_defaults(
            Dynamic.Mission.THRUST_TOTAL, val=22000 * np.ones(2), units="lbf"
        )
        self.prob.model.set_input_defaults(
            Dynamic.Mission.LIFT, val=200 * np.ones(2), units="lbf")
        self.prob.model.set_input_defaults(
            Dynamic.Mission.DRAG, val=10000 * np.ones(2), units="lbf")
        self.prob.model.set_input_defaults(
            Dynamic.Mission.VELOCITY, val=10 * np.ones(2), units="ft/s")
        self.prob.model.set_input_defaults(
            Dynamic.Mission.FLIGHT_PATH_ANGLE, val=np.zeros(2), units="rad")
        self.prob.model.set_input_defaults(Aircraft.Wing.INCIDENCE, val=0, units="deg")
        self.prob.model.set_input_defaults("alpha", val=np.zeros(2), units="deg")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        tol = 1e-6
        self.prob.run_model()

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-12, rtol=1e-12)


if __name__ == "__main__":
    unittest.main()



'''
The following test data comes from a file which was LEAPS2/problems/full_mission_test.py
results. The file does not exist anymore, but the data is used for regression testing.
The analysis converged using SNOPT with 0/1 results and good plots.
One point was taken from each phase: climb, cruise, and descent.

Notes
-----
FLOPS/LEAPS1 mission analyses data cannot be used to unit test Aviary mission
analysis data, because the methodologies are too different. FLOPS/LEAPS1
mission analyses use implementations that do not rely on some Aviary variables,
such as altitude rate and velocity rate. These missing values cannot be derived
from the available FLOPS/LEAPS1 data.
'''
from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.variables import Aircraft, Dynamic, Mission

data = AviaryValues()

data.set_val(
    Mission.Design.GROSS_MASS,
    val=181200.0,
    units='lbm',
)

data.set_val(
    Aircraft.Wing.AREA,
    val=1341,
    units='ft**2',
)

data.set_val(
    Mission.Takeoff.FUEL_SIMPLE,
    val=577,
    units='lbm',
)

data.set_val(
    Mission.Takeoff.LIFT_COEFFICIENT_MAX,
    val=2,
    units='unitless',
)

data.set_val(
    Mission.Takeoff.LIFT_OVER_DRAG,
    val=17.35,
    units='unitless',
)

data.set_val(
    Mission.Takeoff.ROLLING_FRICTION_COEFFICIENT,
    val=.0175,
    units='unitless',
)

data.set_val(
    Mission.Design.THRUST_TAKEOFF_PER_ENG,
    val=24555.5,
    units='lbf',
)

data.set_val(
    # states:altitude
    Dynamic.Mission.ALTITUDE,
    val=[29.3112920637369, 10668, 26.3564405194251, ],
    units='m',
)

data.set_val(
    # outputs
    Dynamic.Mission.ALTITUDE_RATE,
    val=[29.8463233754212, -5.69941245767868E-09, -4.32644785970493, ],
    units='ft/s',
)

data.set_val(
    # outputs
    Dynamic.Mission.ALTITUDE_RATE_MAX,
    val=[3679.0525544843, 3.86361517135375, 6557.07891846677, ],
    units='ft/min',
)

data.set_val(
    # outputs
    Dynamic.Mission.DRAG,
    val=[9978.32211087097, 8769.90342254821, 7235.03338269778, ],
    units='lbf',
)

data.set_val(
    # outputs
    Dynamic.Mission.FUEL_FLOW_RATE,
    val=[16602.302762413, 5551.61304633633, 1286, ],
    units='lbm/h',
)

data.set_val(
    Dynamic.Mission.MACH,
    val=[0.482191004489294, 0.785, 0.345807620281699, ],
    units='unitless',
)

data.set_val(
    # states:mass
    Dynamic.Mission.MASS,
    val=[81796.1389890711, 74616.9849763798, 65193.7423491884, ],
    units='kg',
)

# TODO: double check values
data.set_val(
    Dynamic.Mission.THROTTLE,
    val=[0.5, 0.5, 0., ],
    units='unitless',
)

# TODO: double check values
data.set_val(
    'throttle_max',
    val=[1., 1., 1., ],
    units='unitless',
)

data.set_val(
    # state_rates:range
    Dynamic.Mission.DISTANCE_RATE,
    val=[163.776550884386, 232.775306059091, 117.631414542995, ],
    units='m/s',
)

data.set_val(
    # outputs
    Dynamic.Mission.SPECIFIC_ENERGY_RATE,
    val=[18.4428113202544191, -1.7371801250963E-9, -5.9217623736010768, ],
    units='m/s',
)

data.set_val(
    # outputs
    Dynamic.Mission.SPECIFIC_ENERGY_RATE_EXCESS,
    val=[28.03523893220630, 3.8636151713537548, 28.706899839848, ],
    units='m/s',
)

data.set_val(
    # outputs
    Dynamic.Mission.THRUST_TOTAL,
    val=[30253.9128379374, 8769.90342132054, 0, ],
    units='lbf',
)

data.set_val(
    # outputs
    Dynamic.Mission.THRUST_MAX_TOTAL,
    val=[40799.6009633346, 11500.32, 42308.2709683461, ],
    units='lbf',
)

data.set_val(
    # outputs
    'time',
    val=[85.8098321364695, 8852.1530657591, 23851.1097964271, ],
    units='s',
)

data.set_val(
    # states:velocity
    Dynamic.Mission.VELOCITY,
    val=[164.029012458452, 232.775306059091, 117.638805929526, ],
    units='m/s',
)

data.set_val(
    # state_rates:velocity
    Dynamic.Mission.VELOCITY_RATE,
    val=[0.558739800813549, 3.33665416459715E-17, -0.38372209277242, ],
    units='m/s**2',
)



import numpy as np
import openmdao.api as om

from pathlib import Path

import aviary.constants as constants
from aviary.subsystems.aerodynamics.aero_common import DynamicPressure
from aviary.subsystems.aerodynamics.gasp_based.table_based import CruiseAero as TabularCruiseAero
from aviary.utils.named_values import NamedValues
from aviary.variable_info.variables import Aircraft, Dynamic

grav_metric = constants.GRAV_METRIC_FLOPS


class SolvedAlphaGroup(om.Group):
    """
    Aerodynmaics group that computes the coefficients of lift and drag on a
    structured table of altitude, mach, and angle of attack. The angle of
    attack is solved iteratively to balance the lift with the aircraft weight.

    The aerodynamics data can come from a file or can come from the output of
    a component in the aviary pre_mission group.
    """

    def initialize(self):
        """
        Declare options.
        """
        self.options.declare('num_nodes', types=int)

        self.options.declare('aero_data', types=(str, Path, NamedValues), default=None,
                             desc='Data file or NamedValues object containing lift and '
                                  'drag coefficient table as a function of altitude, '
                                  'Mach, and angle of attack')

        self.options.declare('training_data', default=False,
                             desc='When True, the aero tables will be passed as '
                                  'OpenMDAO variables')

        self.options.declare('structured', types=bool, default=True,
                             desc='Flag that sets if data is a structured grid')

        self.options.declare('extrapolate', default=True, desc='Flag that sets if drag '
                                                               'data can be extrapolated')

    def setup(self):
        options = self.options
        nn = options['num_nodes']
        aero_data = options['aero_data']
        training_data = options['training_data']
        structured = options['structured']
        extrapolate = options['extrapolate']

        self.add_subsystem(
            'DynamicPressure', DynamicPressure(num_nodes=nn),
            promotes_inputs=[Dynamic.Mission.MACH, Dynamic.Mission.STATIC_PRESSURE],
            promotes_outputs=[Dynamic.Mission.DYNAMIC_PRESSURE])

        aero = TabularCruiseAero(num_nodes=nn,
                                 aero_data=aero_data,
                                 training_data=training_data,
                                 structured=structured,
                                 extrapolate=extrapolate)

        if training_data:
            extra_promotes = [Aircraft.Design.DRAG_POLAR,
                              Aircraft.Design.LIFT_POLAR]
        else:
            extra_promotes = []

        self.add_subsystem("tabular_aero", aero,
                           promotes_inputs=[Dynamic.Mission.ALTITUDE, Dynamic.Mission.MACH,
                                            Aircraft.Wing.AREA, Dynamic.Mission.MACH,
                                            Dynamic.Mission.DYNAMIC_PRESSURE]
                           + extra_promotes,
                           promotes_outputs=[Dynamic.Mission.LIFT, Dynamic.Mission.DRAG])

        balance = self.add_subsystem('balance', om.BalanceComp())
        balance.add_balance('alpha', val=np.ones(nn), units='deg', res_ref=1.0e6)

        self.connect('balance.alpha', 'tabular_aero.alpha')
        self.connect('needed_lift.lift_resid', 'balance.lhs:alpha')

        self.add_subsystem('needed_lift',
                           om.ExecComp('lift_resid = mass * grav_metric - computed_lift',
                                       grav_metric={'val': grav_metric},
                                       mass={'units': 'kg', 'shape': nn},
                                       computed_lift={'units': 'N', 'shape': nn},
                                       lift_resid={'shape': nn},
                                       ),
                           promotes_inputs=[('mass', Dynamic.Mission.MASS),
                                            ('computed_lift', Dynamic.Mission.LIFT)]
                           )

        self.linear_solver = om.DirectSolver()
        newton = self.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
        newton.options['iprint'] = 2
        newton.options['atol'] = 1e-9
        newton.options['rtol'] = 1e-12



import unittest

from aviary.examples.variable_meta_data_extension import ExtendedMetaData
from aviary.examples.variables_extension import Aircraft


class MetaDataExtensionTest(unittest.TestCase):
    def test_metadata_extension(self):

        aircraft_variable = ExtendedMetaData[Aircraft.LandingGear.MAIN_GEAR_OLEO_DIAMETER]

        self.assertEqual(Aircraft.LandingGear.MAIN_GEAR_OLEO_DIAMETER,
                         'aircraft:landing_gear:main_gear_oleo_diameter')
        self.assertEqual(aircraft_variable['units'], 'ft')
        self.assertEqual(aircraft_variable['desc'], 'Main gear oleo diameter')
        self.assertEqual(aircraft_variable['default_value'], 0.0)
        self.assertEqual(aircraft_variable['option'], False)
        self.assertEqual(aircraft_variable['types'], None)


if __name__ == '__main__':
    unittest.main()
